# Import the necessary packages
from consolemenu import *
from consolemenu.items import *

# Create the menu
menu = ConsoleMenu("Title", "Subtitle")

# Create some items

# MenuItem is the base class for all items, it doesn't do anything when selected
menu_item = MenuItem("Menu Item")

# A FunctionItem runs a Python function when selected
function_item = FunctionItem("Call a Python function", input, ["Enter an input"])

# A CommandItem runs a console command
command_item = CommandItem("Run a console command",  "touch hello.txt")

# A SelectionMenu constructs a menu from a list of strings
selection_menu = SelectionMenu(["item1", "item2", "item3"])

# A SubmenuItem lets you add a menu (the selection_menu above, for example)
# as a submenu of another menu
submenu_item = SubmenuItem("Submenu item", selection_menu, menu)

# Once we're done creating them, we just add the items to the menu
menu.append_item(menu_item)
menu.append_item(function_item)
menu.append_item(command_item)
menu.append_item(submenu_item)

# Finally, we call show to show the menu and allow the user to interact
menu.show()


# import airsim
import airsim_adaptor

airsim = airsim_adaptor

import setup_path
from pynput.keyboard import Listener, Events
import time
import threading
from multiprocessing import Process, Queue
import pprint
from visca_adapter import viscaAdapter
from sBGC_adapter import sBGCAdaper
import math

# connect to the AirSim simulator
client = airsim.MultirotorClient()
client.confirmConnection()
client.enableApiControl(True)

fpv = "0"
pi = math.pi


class Quaternion:
    def __init__(self, ):
        self.w_val = 0
        self.x_val = 0
        self.y_val = 0
        self.z_val = 0


class Platform:
    def __init__(self):
        self.roll = 0
        self.pitch = 0
        self.yaw = 0
        self.fov = 0
        self.fov_read = 0
        self.value = 0
        self.q = Quaternion()
        self.q1 = Quaternion()
        self.update_pitch = False

    def getCameraInfo(self, camera_):
        cam_info = client.simGetCameraInfo(camera_)
        veh_info = client.simGetVehiclePose("PX4")
        sss = (str(cam_info).split("<CameraInfo>")[1])
        sss1 = sss.split(':')
        self.fov_read = float(sss1[1].split("'")[0].split(",")[0])
        self.q.w_val = float(sss1[4].split(",")[0])
        self.q.x_val = float(sss1[5].split(",")[0])
        self.q.y_val = float(sss1[6].split(",")[0])
        self.q.z_val = float(sss1[7].split(",")[0].split('}')[0])

        ddd = (str(veh_info).split("<Pose>")[1])
        ddd1 = ddd.split(':')
        self.q1.w_val = float(ddd1[2].split(",")[0])
        self.q1.x_val = float(ddd1[3].split(",")[0])
        self.q1.y_val = float(ddd1[4].split(",")[0])
        self.q1.z_val = float(ddd1[5].split(",")[0].split('}')[0])

    def gimbal_update(self, target_pitch):
        self.getCameraInfo(fpv)
        (self.pitch, self.roll, self.yaw) = airsim.to_eularian_angles(self.q)
        (pitch_bd, _, __) = airsim.to_eularian_angles(self.q1)
        pitch_rad = target_pitch*pi/-180
        pitch_bdrad = pitch_bd*pi/-180
        client.simSetCameraPitch(fpv, airsim.to_quaternion(pitch_rad+pitch_bdrad, 0, 0))

    @staticmethod
    def zoom_update(zoom):
        client.simSetCameraFov(fpv, zoom)


if __name__ == '__main__':
    platform = Platform()
    que = Queue()
    que_raw = Queue()
    camera = viscaAdapter()
    gimbal = sBGCAdaper()
    q = Quaternion()
    viscaAdapterT = threading.Thread(target=camera.run)
    viscaAdapterT.start()
    j = False
    sBGCAdapterT = Process(target=gimbal.run, args=(que, que_raw,))
    sBGCAdapterT.start()

    # viscaAdapterI.epoch()
    platform.fov = platform.fov_read
    target_pitch = 0
    while True:
        if not que.empty():
            [platform.update_pitch, target_pitch] = que.get()

        platform.getCameraInfo(fpv)
        (platform.pitch, platform.roll, platform.yaw) = airsim.to_eularian_angles(platform.q)

        if not que_raw.full():
            que_raw.put([platform.pitch, platform.roll, platform.yaw])

        if camera.zoom_update and camera.zoom_fov != 0:
            platform.zoom_update(camera.zoom_fov)
            camera.zoom_update = False

        gimbal.get_raw_orientation(platform.pitch * -180 / pi, platform.roll * -180 / pi, platform.yaw * -180 / pi)
        if platform.update_pitch:
            platform.gimbal_update(target_pitch)
            platform.update_pitch = False








from pynput.keyboard import Listener, Events
import time

class KeyBoard:
    def __init__(self):
        self.keyCommand = ''
        self.dt = 0
        self.i = 1
        self.bounce_time = 0
        self.listener = Listener(on_press=self.get_stroke, on_release=self.delta_time)

    def get_stroke(self, key):
        key_p = str(key)
        if self.i:
            self.dt = time.time()
            self.i = 0

        if 'Key' in key_p:
            key_p = key_p.split('.')[1]
        else:
            key_p = str(key)
        if key_p.lower() == "'w'":
            self.keyCommand = 'p+'
        elif key_p.lower() == "'s'":
            self.keyCommand = 'p-'
        elif key_p.lower() == "'a'":
            self.keyCommand = 'y-'
        elif key_p.lower() == "'a'":
            self.keyCommand = 'y-'
        elif key_p.lower() == "'d'":
            self.keyCommand = 'y+'
        elif key_p.lower() == "'e'":
            self.keyCommand = 'r+'
        elif key_p.lower() == "'q'":
            self.keyCommand = 'r-'
        elif key_p.lower() == 'up':
            self.keyCommand = 'fov-'
        elif key_p.lower() == 'down':
            self.keyCommand = 'fov+'
        else:
            self.keyCommand = ''
        # print(self.keyCommand)

    def delta_time(self, key):
        if not self.i:
            self.bounce_time = self.dt - time.time()
            self.i = 1

    def start_listener(self):
        # listener.join()
        pass



if __name__ == '__main__':
    keyI = KeyBoard()



import serial
import threading
import time
import struct
import timeit
import itertools
import math
from pysimplebgc import SimpleBGC32
from multiprocessing import Queue
SBGC_CMD_START_BYTE = b'\x3e'  # b'>'
SBGC_CMD_CONTROL = b'\x43'  # b'C'
SBGC_CMD_GET_ANGLES = b'\x49'  # b'I'
SBGC_CMD_RESET = b'\x72'  # b'r'
SBGC_FULL_TURN = 16384
SBGC_CONTROL_MODE_ANGLE = b'\x02'

pi = math.pi

class sBGCAdaper:
    def __init__(self):
        self.raw_pitch = 0
        self.raw_roll = 0
        self.raw_yaw = 0
        self.target_pitch = 0
        self.data = []
        self.prev_angle = 0
        self.update_pitch = False
        self.command = []
        self.get_angle = b''
        # self.target_roll  = 100
        # self.target_yaw    = 100
        self.ser = serial.Serial('/dev/tnt3', timeout=2, baudrate=115200)
        time.sleep(1)

        while not self.is_ready(self.ser):
            print("Establishing Connection")
            time.sleep(1)
        print("Connection complete")
        time.sleep(2)
        print(self.ser.name)

    @staticmethod
    def is_ready(ser):
        try:
            ser.write(b'\x00')
            return True
        except:
            return False

    def angle_to_deg(self, angle):
        new_angle = float((360 / SBGC_FULL_TURN) * angle)
        self.target_pitch = round(new_angle * 1.0, ndigits=3)
        if False and (abs(self.prev_angle - self.target_pitch) <= 0.5):
            pass
        else:
            self.update_pitch = True
            # print("Prev. Angle=" + str(self.prev_angle))
            # print("Angle=" + str(self.target_pitch))

    def parser(self):
        times = time.time()
        total_time = 0
        i = -1

        tx_comm = [SBGC_CMD_START_BYTE, SBGC_CMD_GET_ANGLES, b'\x12',
                   b'\x00', b'\x00',
                   b'\x00', b'\x00',
                   b'\x00', b'\x00',

                   b'\x00', b'\x00',
                   b'\x00', b'\x00',
                   b'\x00', b'\x00',

                   b'\x00', b'\x00',
                   b'\x00', b'\x00',
                   b'\x00', b'\x00']

        while True:
            i = i + 1
            elapsed_time = time.time() - times
            times = time.time()
            total_time = total_time + elapsed_time
            if i != 0:
                avg_time = total_time / i
                # print("Time: " + str(avg_time))
                # print("Freq: " + str(1 / avg_time) + "Hz")

            self.command = []
            n = 0
            tx_hb = [SBGC_CMD_START_BYTE, SBGC_CMD_CONTROL, b'\x00', b'\x81', b'\x01', b'\x01', b'\x01']
            for byte in tx_hb:
                self.ser.write(byte)

            if len(self.data):
                data = self.data
                # print("data: " + str(self.data))
                indices = [i for i, x in enumerate(data) if x == b'>']
                # print("indices: " + str(indices))
                if len(indices) > 1:
                    for x in range(len(indices)):
                        if indices[x] != indices[-1]:
                            # print("x:" + str(x))
                            self.command.append(data[indices[x]: indices[x + 1]])
                            empty = [i for i, x in enumerate(self.command[n]) if x == b'']
                            empty.reverse()
                            for z in empty:
                                self.command[n].pop(z)

                            n = n + 1
                # command.sort()
                # list(command for command, _ in itertools.groupby(command))

            for k in range(len(self.command)):
                if len(self.command[k]) >= 2:
                    if self.command[k][1] == SBGC_CMD_GET_ANGLES:
                        pitch = self.deg_to_angle(self.raw_pitch)
                        roll = [b'\x00', b'\x00']  # self.deg_to_angle(self.raw_roll)
                        yaw = [b'\x00', b'\x00']   # self.deg_to_angle(self.raw_yaw)
                        tx_comm[11] = pitch[0]
                        tx_comm[10] = pitch[1]
                        # print(self.raw_pitch)
                        # print(pitch)
                        for byte in tx_comm:
                            self.ser.write(byte)

                    if self.command[k][1] == SBGC_CMD_CONTROL:
                        # print("Command:" + str(command[k]) + "k:" + str(k))
                        self.prev_angle = self.target_pitch
                        if SBGC_CONTROL_MODE_ANGLE in self.command[k] and len(self.command[k]) >= 12:
                            n = self.command[k].index(SBGC_CONTROL_MODE_ANGLE)
                            # if self.checksum_verify(rx_packet[:3]) == rx_packet[len(rx_packet)-1]:
                            # print(rx_packet)

                            self.get_angle = b''.join([self.command[k][n + 7], self.command[k][n + 8]])

    def sBGC_listener(self):
        rx_packet = []
        tx_hb = [SBGC_CMD_START_BYTE, SBGC_CMD_CONTROL, b'\x00', b'\x81', b'\x01', b'\x01', b'\x01']
        for byte in tx_hb:
            self.ser.write(byte)

        temp = self.ser.read()
        if temp == SBGC_CMD_START_BYTE:
            rx_packet.append(temp)
            temp = self.ser.read()
            while not temp == SBGC_CMD_START_BYTE:
                rx_packet.append(temp)
                temp = self.ser.read()

            if len(rx_packet) >= 2:
                if rx_packet[1] == SBGC_CMD_GET_ANGLES:
                    pitch = self.deg_to_angle(self.raw_pitch)
                    roll = self.deg_to_angle(self.raw_roll)
                    yaw = self.deg_to_angle(self.raw_yaw)

                    tx_comm = [SBGC_CMD_START_BYTE, SBGC_CMD_GET_ANGLES, b'\x12',
                               b'\x00', b'\x00',
                               roll[1], roll[0],
                               b'\x00', b'\x00',
                               b'\x00', b'\x00',
                               pitch[1], pitch[0],
                               b'\x00', b'\x00',
                               b'\x00', b'\x00',
                               yaw[1], yaw[0],
                               b'\x00', b'\x00']
                    for byte in tx_comm:
                        self.ser.write(byte)
                if rx_packet[1] == SBGC_CMD_CONTROL:
                    if SBGC_CONTROL_MODE_ANGLE in rx_packet and len(rx_packet) >= 12:
                        n = rx_packet.index(SBGC_CONTROL_MODE_ANGLE)
                        # if self.checksum_verify(rx_packet[:3]) == rx_packet[len(rx_packet)-1]:
                        # print(rx_packet)
                        angle = b''.join([rx_packet[n + 7], rx_packet[n + 8]])
                        self.angle_to_deg(struct.unpack('<h', angle)[0])
        # for byte in tx_hb:
        #    self.ser.write(byte)

    @staticmethod
    def deg_to_angle(deg):
        d = int((SBGC_FULL_TURN / 360) * deg)
        # print("Angle =" + str(d))
        angle = []
        d_h = struct.pack('>h', d)
        for D in d_h:
            angle.append(D.to_bytes(1, byteorder="little"))
        return angle

    @staticmethod
    def calculate_h_checksum(header):
        return bytes((header[1] + header[2]) % 256)

    def run(self, que, que_raw):
        listenerT = threading.Thread(target=self.listener)
        listenerT.start()
        parserT = threading.Thread(target=self.parser)
        parserT.start()

        while True:
            if not que_raw.empty():
                raw_data = que_raw.get()
                self.raw_pitch = raw_data[0]*-180/pi
                self.raw_roll = raw_data[1]*-180/pi
                self.raw_yaw = raw_data[2]*-180/pi
                if not que_raw.empty():
                    raw_data = que_raw.get()
                    self.raw_pitch = raw_data[0] * -180 / pi
                    self.raw_roll = raw_data[1] * -180 / pi
                    self.raw_yaw = raw_data[2] * -180 / pi

            if self.get_angle != b'':
                self.prev_angle = self.target_pitch
                self.angle_to_deg(struct.unpack('<h', self.get_angle)[0])
                # self.get_angle = b''

                if self.update_pitch:
                    diff = abs(self.target_pitch - self.prev_angle)
                    # if diff != 0: print(diff)
                    if diff > 0.01:
                        if not que.full():
                            que.put([self.update_pitch, self.target_pitch])
                        # else:
                        # print("full")
                self.update_pitch = False

    def get_raw_orientation(self, p, r, y):
        self.raw_pitch = p
        self.raw_roll = r
        self.raw_yaw = y

    def set_orientation(self, p, r, y):
        return p, r, y

    def listener(self):
        while True:
            self.data.append(self.ser.read())

            if len(self.data) > 100:
                self.data = []


if __name__ == '__main__':
    gimbalI = sBGCAdaper()
    gimbalI.run()



# Import this module to automatically setup path to local airsim module
# This module first tries to see if airsim module is installed via pip
# If it does then we don't do anything else
# Else we look up grand-parent folder to see if it has airsim folder
#    and if it does then we add that in sys.path

import os, sys, logging

#this class simply tries to see if airsim 
class SetupPath:
    @staticmethod
    def getDirLevels(path):
        path_norm = os.path.normpath(path)
        return len(path_norm.split(os.sep))

    @staticmethod
    def getCurrentPath():
        cur_filepath = __file__
        return os.path.dirname(cur_filepath)

    @staticmethod
    def getGrandParentDir():
        cur_path = SetupPath.getCurrentPath()
        if SetupPath.getDirLevels(cur_path) >= 2:
            return os.path.dirname(os.path.dirname(cur_path))
        return ''

    @staticmethod
    def getParentDir():
        cur_path = SetupPath.getCurrentPath()
        if SetupPath.getDirLevels(cur_path) >= 1:
            return os.path.dirname(cur_path)
        return ''

    @staticmethod
    def addAirSimModulePath():
        # if airsim module is installed then don't do anything else
        #import pkgutil
        #airsim_loader = pkgutil.find_loader('airsim')
        #if airsim_loader is not None:
        #    return

        parent = SetupPath.getParentDir()
        if parent !=  '':
            airsim_path = os.path.join(parent, 'airsim')
            client_path = os.path.join(airsim_path, 'client.py')
            if os.path.exists(client_path):
                sys.path.insert(0, parent)
        else:
            logging.warning("airsim module not found in parent folder. Using installed package (pip install airsim).")

SetupPath.addAirSimModulePath()



import serial
import time
import threading



VISCA_CAM_STABILIZER    = b'4'
VISCA_ZOOM_VALUE        = b'G'
VISCA_CATEGORY_IF       = b'\x00'
VISCA_CMD               = b'\x01'
VISCA_CAM_STABILIZER_ON = b'\x02'
VISCA_CAM_STABILIZER_OFF= b'\x03'
VISCA_CATEGORY_CAMERA1  = b'\x04'
VISCA_INQUIRY           = b'\x10'
VISCA_RSP               = b'\x81'
VISCA_CALL              = b'\x88'
VISCA_TERMINATOR        = b'\xff'

VENDOR_H    = b'S'
VENDOR_L    = b'T'
MODEL_H     = b'M'
MODEL_L     = b' '
ROM_H       = b'V'
ROM_L       = b'1'
DEV_SOCK    = b'\x01'
ADP_ADDR    = b'\x02'
HEAD        = b'\x80'

STABILIZER_INIT = [VISCA_RSP, VISCA_CMD, VISCA_CATEGORY_CAMERA1, VISCA_CAM_STABILIZER, VISCA_CAM_STABILIZER_ON, VISCA_TERMINATOR]


class viscaAdapter:
    def __init__(self, ):
        self.zoom_fov = 0
        self.epoch_flag = True
        self.zoom_update = False
        self.ser = serial.Serial('/dev/tnt1', timeout=2)
        print(self.ser.name)

        self.zoom_map_list = {
            # 0: 110,
            # 262144: 100,
            # 524288: 90,
            # 786432: 80,
            # 16777216: 70,
            # 17039360: 60,
            # 17563648: 50,
            # 33554432: 40,
            # 34078720: 30,
            # 50331648: 20,
            # 50855936: 10,
            # 67108864: 1

            0: 65.9155922,
            1024: 59.0054079,
            2048: 52.9629756,
            3072: 47.2879077,
            4096: 42.1146225,
            5120: 37.3802374,
            6144: 33.2133223,
            7168: 29.1496556,
            8192: 25.340908,
            9216: 22.0776279,
            10240: 18.8118776,
            11264: 15.9241344,
            12288: 13.4961993,
            13312: 11.2971676,
            14336: 9.55587665,
            15360: 7.9487979,
            16384: 6.7890118
                        }

    def wait_for_call(self):
        rx_packet = []
        temp = self.ser.read()
        if temp == VISCA_CALL:
            rx_packet.append(VISCA_CALL)
            while not temp == VISCA_TERMINATOR:
                temp = self.ser.read()
                rx_packet.append(temp)
        elif temp == VISCA_RSP:
            self.visca_listener()
        if rx_packet == [VISCA_CALL, b'\x30', b'\x01', VISCA_TERMINATOR]:
            tx_comm = rx_packet
            tx_comm[2] = ADP_ADDR
            for byte in tx_comm:
                self.ser.write(byte)
                # print(byte)
            self.set_addr_rsp()

    def set_addr_rsp(self):
        rx_packet = []
        temp = self.ser.read()
        if temp == VISCA_RSP:
            rx_packet.append(VISCA_RSP)
            while not temp == VISCA_TERMINATOR:
                temp = self.ser.read()
                rx_packet.append(temp)
            # print(rx_packet)
        if rx_packet == [VISCA_RSP, VISCA_CMD, VISCA_CATEGORY_IF, b'\x01', VISCA_TERMINATOR]:
            tx_comm = [VISCA_RSP, b'\x50', VISCA_TERMINATOR]
            for byte in tx_comm:
                self.ser.write(byte)
                # print(byte)
            self.device_inquiry_rsp()

    def device_inquiry_rsp(self):
        rx_packet = []
        temp = self.ser.read()
        if temp == VISCA_RSP:
            rx_packet.append(VISCA_RSP)
            while not temp == VISCA_TERMINATOR:
                temp = self.ser.read()
                rx_packet.append(temp)
        if rx_packet == [VISCA_RSP, b'\t', VISCA_CATEGORY_IF, b'\x02', VISCA_TERMINATOR]:
            tx_comm = [VISCA_RSP, b'\x50', VENDOR_H, VENDOR_L, MODEL_H, MODEL_L, ROM_H, ROM_L, DEV_SOCK, VISCA_TERMINATOR]
            for byte in tx_comm:
                self.ser.write(byte)
        self.visca_listener()

    def visca_listener(self):
        while True:
            rx_packet = []
            temp = self.ser.read()
            if not temp == b'':
                rx_packet.append(temp)
            while not temp == VISCA_TERMINATOR or temp == VISCA_RSP:
                temp = self.ser.read()
                rx_packet.append(temp)
            # print(rx_packet)
            if VISCA_CMD in rx_packet:
                if VISCA_CAM_STABILIZER in rx_packet:
                    n = rx_packet.index(VISCA_CAM_STABILIZER)
                    if rx_packet[n+1] == VISCA_CAM_STABILIZER_ON:
                        print("Stabilizer Enabled")
                    else:
                        print("Stabilizer not Enabled")
                    if rx_packet == STABILIZER_INIT:
                        print("Stabilizer Initialized")

                elif VISCA_ZOOM_VALUE in rx_packet:
                    n = rx_packet.index(VISCA_ZOOM_VALUE)
                    # zoom_bytes = b''.join([rx_packet[n+1], rx_packet[n+2], rx_packet[n+3], rx_packet[n+4]])
                    zoom_bytes = [rx_packet[n+1], rx_packet[n+2], rx_packet[n+3], rx_packet[n+4]]
                    zoom = 0
                    for i in range(4):
                        zoom = zoom + int.from_bytes(zoom_bytes[3-i], byteorder='little')*pow(16, i)
                    # for byte in zoom_bytes:
                    #     self.zoom = self.zoom + byte,
                    self.zoom_to_fov(zoom)
                else:
                    print("Unknown Command {}".format(rx_packet))

            tx_comm = [VISCA_RSP, b'\x50', VENDOR_H, VENDOR_L, MODEL_H, MODEL_L, ROM_H, ROM_L, DEV_SOCK,
                       VISCA_TERMINATOR]
            for byte in tx_comm:
                self.ser.write(byte)

    def zoom_to_fov(self, zoom):
        mapped_key = min(self.zoom_map_list, key=lambda x: abs(x-zoom))
        self.zoom_fov = self.zoom_map_list[mapped_key]
        # print("Fov:" + str(self.zoom_fov))
        self.zoom_update = True

    def bytes_to_int(self, byte):
        result = 0
        for b in byte:
            result = result * 256 + int(b)
        return result

    def epoch(self):
        self.epoch_flag = True
        threading.Timer(2, self.epoch).start()

    def run(self):
        # self.epoch()
        while True:
            self.wait_for_call()


if __name__ == '__main__':
    viscaAdapterI = viscaAdapter()
    viscaAdapterI.epoch()
    viscaAdapterI.run()
